@@ -20,7 +20,7 @@ application:
 from itertools import chain
 from pathlib import Path
 
-from skrt.core import Defaults, fullpath, get_logger
+from skrt.core import Defaults, fullpath, get_logger, is_list
 from skrt.patient import Patient
 
 
@@ -235,6 +235,7 @@ class Application():
                 patient = PatientClass(path=data_path, **kwargs)
                 self.status = self.execute(patient=patient)
                 if not self.status.ok():
+                    self.logger.error(self.status)
                     break
 
         if self.status.ok():
@@ -250,6 +251,7 @@ class Status():
     **Methods:**
 
     * **__init__()**: Create instance of Status class.
+    * **copy_attributes()**: Copy attributes from another Status object.
     * **ok()**: Return boolean, indicating whether status is okay.
     '''
 
@@ -275,6 +277,10 @@ class Status():
         self.name = name or ""
         self.reason = reason or ""
 
+    def __repr__(self):
+        """Print status information."""
+        return f"Status {self.code} ({self.name}): {self.reason}"
+
     def ok(self):
         '''
         Return boolean indicating whether status is okay (exit code non-zero).
@@ -286,8 +292,22 @@ class Status():
 
         return is_ok
 
+    def copy_attributes(self, status):
+        '''
+        Copy attributes from another Status object.
 
-def get_paths(data_locations=None, max_path=None, ids=None):
+        **Parameter:**
+
+        status : skrt.application.Status
+            Status object from which to copy attributes.
+        '''
+        self.code = status.code
+        self.name = status.name
+        self.reason = status.reason
+
+
+def get_paths(data_locations=None, max_path=None,
+        to_keep=None, to_exclude=None):
     """
     Get list of paths to patient datasets.
 
@@ -296,7 +316,7 @@ def get_paths(data_locations=None, max_path=None, ids=None):
     data_locations : pathlib.Path/str/list/dict, default=None
         Specification of how to identify paths to patient datasets:
 
-        - string pathlib.Path giving path to a single patient dataset;
+        - string of pathlib.Path giving path to a single patient dataset;
         - list of strings or pathlib.Path giving full paths to patient datasets;
         - dictionary where keys are paths to directories containing
           patient datasets, and values are strings, or lists of strings,
@@ -306,9 +326,15 @@ def get_paths(data_locations=None, max_path=None, ids=None):
         Maximum number of paths to return.  If None, return paths to all
         directories in data directory (currently hardcoded in this function).
 
-    ids : str/list, default=None
+    to_keep : str/list, default=None
         Patient identifier, or list of patient identifiers.  If non-null,
         only return dataset paths for the specified identifier(s).
+
+    to_exclude : str/list, default=None
+        Patient identifier, or list of patient identifiers.  If non-null,
+        only return dataset paths excluding the specified identifier(s).
+        If an identifier is specified as both <to_keep> and <to_exclude>,
+        the latter takes precedence.
     """
     # Return empty list if data_locations is None.n
     paths = []
@@ -332,11 +358,17 @@ def get_paths(data_locations=None, max_path=None, ids=None):
                 if path.is_dir()])
 
     # Filter list of dataset paths.
-    if ids:
-        if not is_list(ids):
-            ids = [ids]
+    if to_keep:
+        if not is_list(to_keep):
+            to_keep = [to_keep]
         paths = [path for path in paths
-                if any([Path(path).match(id) for id in ids])]
+                if any([Path(path).match(id) for id in to_keep])]
+
+    if to_exclude:
+        if not is_list(to_exclude):
+            to_exclude = [to_exclude]
+        paths = [path for path in paths
+                if not any([Path(path).match(id) for id in to_exclude])]
 
     # Sort paths, ensuring that each path is included only once, 
     paths = sorted(list(set(paths)))
