@@ -1,12 +1,15 @@
 """Tools for performing image segmentation."""
 from concurrent.futures import ThreadPoolExecutor
+from inspect import signature
+from itertools import combinations
 from pathlib import Path
 from shutil import rmtree
 
 import pandas as pd
 
 from skrt.core import (
-        get_logger, get_stat_functions, is_list, Data, Defaults, tic, toc)
+        get_dict_permutations, get_logger, get_stat_functions, is_list,
+        Data, Defaults, tic, toc)
 from skrt.image import match_images, Image
 from skrt.structures import (
         get_by_slice_methods, get_comparison_metrics, get_consensus_types,
@@ -18,7 +21,8 @@ class MultiAtlasSegmentation(Data):
         self, im1=None, im2=None, ss1=None, ss2=None, log_level=None,
         workdir="segmentation_workdir", overwrite=False,
         auto=False, auto_step=None, auto_strategies=None,
-        default_step=-1, default_strategy="pull", roi_names=None,
+        auto_reg_setup_only=False,
+        default_step=None, default_strategy="pull", roi_names=None,
         metrics=None, slice_stats=None, default_by_slice=None,
         consensus_types="majority", default_consensus_type="majority",
         max_workers=1, **kwargs):
@@ -74,6 +78,7 @@ class MultiAtlasSegmentation(Data):
         self.default_consensus_type = get_options(
             default_consensus_type, self.consensus_types,
             get_consensus_types())[-1]
+        self.consensus_atlas_ids = set()
 
         # Set maximum number of processes when multiprocessing.
         self.max_workers = max_workers
@@ -89,11 +94,12 @@ class MultiAtlasSegmentation(Data):
         # Perform segmentation.
         if auto:
             for auto_strategy in self.auto_strategies:
-                self.segment(strategy=auto_strategy, step=self.auto_step)
+                self.segment(strategy=auto_strategy, step=self.auto_step,
+                             reg_setup_only=auto_reg_setup_only)
 
     def segment(self, atlas_ids=None, strategy=None, step=None,
-            consensus_types=None, force=False):
-        atlas_ids = atlas_ids or self.sass.keys()
+            consensus_types=None, force=False, reg_setup_only=False):
+        atlas_ids = get_options(atlas_ids, True, list(self.sass))
         strategy = get_option(strategy, self.default_strategy, self.strategies)
         steps = get_steps(step)
 
@@ -102,14 +108,14 @@ class MultiAtlasSegmentation(Data):
         sas_auto = True
         for idx in atlas_ids:
             if ((self.sass[idx] is None) or (force) or
-                    any([self.sass[idx].segmentations[strategy][step] is None
+                    any([not self.sass[idx].segmentations[strategy][step]
                         for step in steps])):
                 active_ids.append(idx)
                 args = (self.im1, self.im2[idx], self.ss1, self.ss2[idx],
                         self.log_level, self.workdir / str(idx), self.overwrite,
                         sas_auto, self.auto_step, self.auto_strategies,
-                        self.default_step, self.default_strategy,
-                        self.roi_names,)
+                        reg_setup_only, self.default_step,
+                        self.default_strategy, self.roi_names,)
                 if self.max_workers == 1:
                     tic()
                     self.logger.info(f"Segmenting with atlas '{idx}'")
@@ -137,29 +143,37 @@ class MultiAtlasSegmentation(Data):
             for idx in futures:
                 self.sass[idx] = futures[idx].result()
 
+        if reg_setup_only:
+            return
+
         for idx in active_ids:
             self.sass[idx].segment(strategy, step, force)
             for step in steps:
-                reg = self.sass[idx].segmentations[strategy][step]
-                for reg_step in reg:
-                    reg[reg_step].name = f"{idx}_{reg[reg_step].name}"
+                seg = self.sass[idx].segmentations[strategy][step]
+                for reg_step in seg:
+                    seg[reg_step].name = f"{idx}_{seg[reg_step].name}"
 
         if consensus_types is None:
             consensus_types = self.consensus_types
         if consensus_types:
             for step in steps:
-                for reg_step in self.get_sas().segmentations[strategy][step]:
-                    self.set_consensuses(
-                            strategy, step, reg_step, consensus_types, force)
+                for reg_step in self.get_sas(atlas_ids[0]).get_reg_steps(step):
+                    self.set_consensuses(atlas_ids, strategy, step, reg_step,
+                                         consensus_types, force)
 
-    def get_consensus(self, strategy=None, step=None, reg_step=None,
-            consensus_type=None, force=False):
+    def get_consensus(self, atlas_ids=None, strategy=None, step=None,
+                      reg_step=None, consensus_type=None, force=False):
+        atlas_ids = get_options(atlas_ids, True, list(self.sass))
         strategy = get_option(strategy, self.default_strategy, self.strategies)
         step = get_option(step, self.auto_step, self.steps)
         consensus_type = consensus_type or self.default_consensus_type
-        self.segment(None, strategy, step, consensus_type, force)
-        reg_steps = list(self.get_sas().segmentations[strategy][step])
-        reg_step = get_option(reg_step, None, reg_steps)
+        if self.consensus_atlas_ids != set(atlas_ids):
+            self.segment(atlas_ids, strategy, step, consensus_type, force)
+        all_reg_steps = self.get_sas(atlas_ids[0]).get_reg_steps(step)
+        reg_step = get_option(reg_step, None, all_reg_steps)
+        if ((reg_step not in self.consensuses[strategy][step]) or
+            (consensus_type not in self.consensuses[strategy][step][reg_step])):
+            self.segment(atlas_ids, strategy, step, consensus_type, force)
         return self.consensuses[strategy][step][reg_step][consensus_type]
 
     def get_sas(self, atlas_id=None):
@@ -176,8 +190,8 @@ class MultiAtlasSegmentation(Data):
             strategy, step, reg_step, force) for atlas_id in atlas_ids],
             StructureSet())
 
-    def set_consensuses(self, strategy=None, step=None, reg_step=None,
-            consensus_types=None, force=False):
+    def set_consensuses(self, atlas_ids=None, strategy=None, step=None,
+                        reg_step=None, consensus_types=None, force=False):
 
         if consensus_types is None:
             consensus_types = self.consensus_types
@@ -186,24 +200,33 @@ class MultiAtlasSegmentation(Data):
         if isinstance(consensus_types, str):
             consensus_types = [consensus_types]
 
+        atlas_ids = get_options(atlas_ids, True, list(self.sass))
         strategy = get_option(strategy, self.default_strategy, self.strategies)
         step = get_option(step, self.auto_step, self.steps)
-        reg_steps = list(self.get_sas().segmentations[strategy][step])
-        reg_step = get_option(reg_step, None, reg_steps)
+        all_reg_steps = self.get_sas(atlas_ids[0]).get_reg_steps(step)
+        reg_step = get_option(reg_step, None, all_reg_steps)
         if not reg_step in self.consensuses[strategy][step]:
             self.consensuses[strategy][step][reg_step] = {}
 
         all_rois = {roi_name: [] for roi_name in self.roi_names}
-        for sas in self.sass.values():
+        for atlas_id in atlas_ids:
+            sas = self.get_sas(atlas_id)
             ss = sas.get_segmentation(strategy, step, reg_step)
             for roi in ss.get_rois():
                 if roi.name in all_rois:
                     all_rois[roi.name].append(roi)
 
+        atlas_ids = set(atlas_ids)
         for consensus_type in consensus_types:
             if (consensus_type in self.consensuses[strategy][step][reg_step]
-                    and not force):
+                and (not force) and (self.consensus_atlas_ids == atlas_ids)):
                 continue
+
+            if self.consensus_atlas_ids != atlas_ids:
+                for tmp_reg_step in self.consensuses[strategy][step]:
+                    self.consensuses[strategy][step][tmp_reg_step] = {}
+                self.consensus_atlas_ids = atlas_ids
+
             ss_consensus = StructureSet(
                     name=f"{strategy}_{step}_{reg_step}_{consensus_type}")
             for roi_name, rois in all_rois.items():
@@ -217,15 +240,28 @@ class MultiAtlasSegmentation(Data):
                     ss_consensus)
 
     def get_comparison(
-            self, consensus_types=None, atlas_ids=False,
+            self, consensus_types=None, atlas_ids=True,
             id1=None, id2=None, to_keep=None, strategies=None,
             steps=None, reg_steps=None, force=False, metrics=None,
-            slice_stats=None, default_by_slice=None, 
-            voxel_size=None, name_as_index=False, **kwargs):
+            slice_stats=None, default_by_slice=None, voxel_size=None,
+            name_as_index=False, atlas_ids_to_compare=False,
+            combination_length=None, **kwargs):
+
         consensus_types = get_options(
                 consensus_types, self.consensus_types, get_consensus_types())
-        n_consensus_type = len(consensus_types)
-        atlas_ids = get_options(atlas_ids, None, list(self.sass.keys()))
+        if not (is_list(atlas_ids) and atlas_ids and is_list(atlas_ids[0])):
+            atlas_ids = get_options(atlas_ids, None, list(self.sass))
+            if (combination_length or 0) > 1:
+                atlas_ids = list(combinations(atlas_ids, combination_length))
+            else:
+                atlas_ids = [atlas_ids]
+
+        start = tuple() if isinstance(atlas_ids[0], tuple) else list()
+        all_atlas_ids = list(set(sum(atlas_ids, start)))
+
+        atlas_ids_to_compare = get_options(
+            atlas_ids_to_compare, None, all_atlas_ids)
+        n_atlas_ids_to_compare = len(atlas_ids_to_compare)
 
         strategies = get_options(
                 strategies, self.default_strategy, self.strategies)
@@ -236,52 +272,68 @@ class MultiAtlasSegmentation(Data):
                                   get_stat_functions())
         default_by_slice = get_option(default_by_slice, self.default_by_slice,
                                       get_by_slice_methods())
+
+        sas_id = get_options(atlas_ids_to_compare, True, all_atlas_ids)[0]
+        sas = self.get_sas(sas_id)
+        step_reg_steps = {}
+        for step in steps:
+            all_reg_steps = sas.get_reg_steps(step)
+            step_reg_steps[step] = get_options(reg_steps, True, all_reg_steps)
         
         df = None
-        ss1 = self.get_sas().ss1_filtered.filtered_copy(to_keep=to_keep)
-        for idx, id2 in enumerate(consensus_types + atlas_ids):
+        ss1 = sas.ss1_filtered.filtered_copy(to_keep=to_keep)
+        for idx, id2 in enumerate(atlas_ids_to_compare + consensus_types):
+            if idx < n_atlas_ids_to_compare:
+                sas = self.get_sas(id2)
             for strategy in strategies:
                 for step in steps:
-                    if idx < n_consensus_type:
-                        self.segment(None, strategy, step, id2, force)
-                        all_reg_steps = list(self.consensuses[strategy][step])
-                    else:
-                        sas = self.get_sas(id2)
-                        sas.segment(strategy, step, force)
-                        all_reg_steps = list(sas.segmentations[strategy][step])
-
-                    step_reg_steps = get_options(reg_steps, None, all_reg_steps)
-                    for reg_step in step_reg_steps:
-                        if idx < n_consensus_type:
-                            ss2 = (self.get_consensus(
-                                strategy, step, reg_step, consensus_type=id2)\
-                                        .filtered_copy(to_keep=to_keep))
+                    for reg_step in step_reg_steps[step]:
+                        if idx < n_atlas_ids_to_compare:
+                            tic()
+                            ss2s = {id2: (sas.get_segmentation(
+                                strategy, step, reg_step
+                                ).filtered_copy(to_keep=to_keep))}
+                            self.logger.info(
+                                    f"{id2} [{strategy}, {step}, {reg_step}]: "
+                                    f"{toc():.2f} s")
                         else:
-                            ss2 = (sas.get_segmentation(
-                                strategy, step, reg_step)\
-                                        .filtered_copy(to_keep=to_keep))
-                        df_tmp = ss1.get_comparison(
-                                ss2, metrics=metrics, slice_stats=slice_stats,
-                                default_by_slice=default_by_slice,
-                                voxel_size=voxel_size,
-                                name_as_index=name_as_index,
-                                **kwargs)
+                            ss2s = {}
+                            for tmp_atlas_ids in atlas_ids:
+                                label = "_".join([str(atlas_id)
+                                                  for atlas_id in tmp_atlas_ids]
+                                                 + [id2])
+                                ss2s[label] = (self.get_consensus(tmp_atlas_ids,
+                                    strategy, step, reg_step, consensus_type=id2
+                                    ).filtered_copy(to_keep=to_keep))
+                            self.logger.info(
+                                    f"{label} "
+                                    f"[{strategy}, {step}, {reg_step}]: "
+                                    f"{toc():.2f} s")
 
-                        if df_tmp is None:
-                            continue
+                        for id2_label, ss2 in ss2s.items():
+                            df_tmp = ss1.get_comparison(
+                                    ss2, metrics=metrics,
+                                    slice_stats=slice_stats,
+                                    default_by_slice=default_by_slice,
+                                    voxel_size=voxel_size,
+                                    name_as_index=name_as_index,
+                                    **kwargs)
 
-                        for label, value in [
-                                ("id1", id1), ("id2", id2),
-                                ("strategy", strategy), ("step", step),
-                                ("reg_step", reg_step)]:
-                            if value is not None:
-                                df_tmp[label] = pd.Series(
-                                        df_tmp.shape[0] * [value])
+                            if df_tmp is None:
+                                continue
 
-                        if df is None:
-                            df = df_tmp
-                        else:
-                            df = pd.concat([df, df_tmp], ignore_index=True)
+                            for label, value in [
+                                    ("id1", id1), ("id2", id2_label),
+                                    ("strategy", strategy), ("step", step),
+                                    ("reg_step", reg_step)]:
+                                if value is not None:
+                                    df_tmp[label] = pd.Series(
+                                            df_tmp.shape[0] * [value])
+
+                            if df is None:
+                                df = df_tmp
+                            else:
+                                df = pd.concat([df, df_tmp], ignore_index=True)
 
         return df
 
@@ -289,10 +341,11 @@ class MultiAtlasSegmentation(Data):
             self, atlas_ids=None, strategy=None, step=None, roi_name=None,
             reg_step=None, force=False, max_keep=None, **kwargs):
 
-        atlas_ids = get_options(atlas_ids, True, list(self.sass.keys()))
+        atlas_ids = get_options(atlas_ids, True, list(self.sass))
         scores1 = {}
         for atlas_id in atlas_ids:
-            self.sass[atlas_id].segment(strategy, step, force)
+            self.sass[atlas_id].segment(
+                    strategy, step, force, reg_setup_only=True)
             reg = self.sass[atlas_id].get_registration(
                     strategy, step, roi_name, force)
             score = reg.get_mutual_information(reg_step, **kwargs)
@@ -316,6 +369,7 @@ class SingleAtlasSegmentation(Data):
         self, im1=None, im2=None, ss1=None, ss2=None, log_level=None,
         workdir="segmentation_workdir", overwrite=False,
         auto=False, auto_step=None, auto_strategies=None,
+        auto_reg_setup_only=False,
         default_step=-1, default_strategy="pull", roi_names=None,
         ss1_index=-1, ss2_index=-1, ss1_name="Filtered1", ss2_name="Filtered2",
         initial_crop_focus=None, initial_crop_margins=None,
@@ -323,7 +377,8 @@ class SingleAtlasSegmentation(Data):
         crop_to_match_size1=True, voxel_size1=None, bands1=None, pfiles1=None, 
         most_points1=True, roi_crop_margins=None, default_roi_crop_margins=10,
         roi_pfiles=None, crop_to_match_size2=True, voxel_size2=None,
-        bands2=None, pfiles2=None, most_points2=True, capture_output=False,
+        default_roi_bands=None, roi_bands=None, pfiles2=None,
+        most_points2=True, capture_output=False,
         keep_tmp_dir=False, metrics=None, slice_stats=None,
         default_by_slice=None):
 
@@ -378,7 +433,8 @@ class SingleAtlasSegmentation(Data):
         self.default_roi_crop_margins = default_roi_crop_margins
         self.crop_to_match_size2 = crop_to_match_size2
         self.voxel_size2 = voxel_size2
-        self.bands2 = bands2
+        self.roi_bands = roi_bands or {}
+        self.default_roi_bands = default_roi_bands
         self.pfiles2 = pfiles2 or pfiles1
         self.pfiles2_non_null = {reg_step: pfile for reg_step, pfile
                                  in (self.pfiles2 or {}).items() if pfile}
@@ -408,72 +464,21 @@ class SingleAtlasSegmentation(Data):
         # Perform segmentation.
         if auto:
             for auto_strategy in self.auto_strategies:
-                self.segment(auto_strategy, self.auto_step)
+                self.segment(auto_strategy, self.auto_step,
+                             reg_setup_only=auto_reg_setup_only)
 
-    def segment(self, strategy=None, step=None, force=False):
+    def segment(self, strategy=None, step=None, force=False,
+                reg_setup_only=False):
         strategy = get_option(strategy, self.default_strategy, self.strategies)
         steps = get_steps(step)
 
         if force:
-            self.registrations[strategy] = {step: {} for step in steps}
-            self.segmentations[strategy] = {step: {} for step in steps}
+            for step in steps:
+                self.registrations[strategy][step] = {}
+                self.segmentations[strategy][step] = {}
 
-        if (self.steps[0] in steps
-                and not self.registrations[strategy][self.steps[0]]):
-            im1, im2 = match_images(
-                    im1=self.im1,
-                    im2=self.im2,
-                    ss1_index = self.ss1_index,
-                    ss2_index = self.ss2_index,
-                    ss1=self.ss1,
-                    ss2=self.ss2,
-                    ss1_name=self.ss1_name,
-                    ss2_name=self.ss2_name,
-                    roi_names=self.roi_names,
-                    im2_crop_focus=self.initial_crop_focus,
-                    im2_crop_margins=self.initial_crop_margins,
-                    alignment=(self.initial_alignment
-                               if self.crop_to_match_size1 else False),
-                    voxel_size=self.voxel_size1,
-                    bands=self.bands1)
-
-            self.ss1_filtered = im1.structure_sets[0]
-            self.ss2_filtered = im2.structure_sets[0]
-
-            fixed, moving = get_fixed_and_moving(im1, im2, strategy)
-
-            self.registrations[strategy][self.steps[0]] = (
-                    Registration(
-                        self.workdir / f"{strategy}1",
-                        fixed=fixed,
-                        moving=moving,
-                        pfiles=self.pfiles1,
-                        initial_alignment=self.initial_alignment,
-                        initial_transform_name=self.initial_transform_name,
-                        capture_output=self.capture_output,
-                        log_level=self.log_level,
-                        keep_tmp_dir=self.keep_tmp_dir,
-                        overwrite=force,
-                        )
-                    )
-
-            self.segment_structure_set(im2.structure_sets[0],
-                    strategy, self.steps[0], self.most_points1)
-
-        if (self.steps[1] in steps
-                and not self.registrations[strategy][self.steps[1]]):
-
-            pfiles2 = {}
-            if self.initial_alignment:
-                self.segmentations[strategy][self.steps[1]]\
-                        [self.initial_transform_name] = {}
-            for reg_step, pfile in self.pfiles2.items():
-                self.segmentations[strategy][self.steps[1]][reg_step] = {}
-                if pfile:
-                    pfiles2[reg_step] = pfile
-
-            rois = []
-            for roi_name in self.roi_names:
+        if self.steps[0] in steps:
+            if not self.registrations[strategy][self.steps[0]]:
                 im1, im2 = match_images(
                         im1=self.im1,
                         im2=self.im2,
@@ -483,24 +488,26 @@ class SingleAtlasSegmentation(Data):
                         ss2=self.ss2,
                         ss1_name=self.ss1_name,
                         ss2_name=self.ss2_name,
-                        roi_names={roi_name: self.roi_names[roi_name]},
-                        im2_crop_focus=roi_name,
-                        im2_crop_margins=self.roi_crop_margins.get(
-                            roi_name, self.default_roi_crop_margins),
-                        alignment=(roi_name if self.crop_to_match_size2
-                                   else False),
-                        voxel_size=self.voxel_size2,
-                        bands=self.bands2)
+                        roi_names=self.roi_names,
+                        im2_crop_focus=self.initial_crop_focus,
+                        im2_crop_margins=self.initial_crop_margins,
+                        alignment=(self.initial_alignment
+                                   if self.crop_to_match_size1 else False),
+                        voxel_size=self.voxel_size1,
+                        bands=self.bands1)
+
+                self.ss1_filtered = im1.structure_sets[0]
+                self.ss2_filtered = im2.structure_sets[0]
 
                 fixed, moving = get_fixed_and_moving(im1, im2, strategy)
 
-                self.registrations[strategy][self.steps[1]][roi_name] = (
+                self.registrations[strategy][self.steps[0]] = (
                         Registration(
-                            self.workdir / f"{strategy}2" / roi_name,
+                            self.workdir / f"{strategy}1",
                             fixed=fixed,
                             moving=moving,
-                            pfiles=self.roi_pfiles.get(roi_name, pfiles2),
-                            initial_alignment=roi_name,
+                            pfiles=self.pfiles1,
+                            initial_alignment=self.initial_alignment,
                             initial_transform_name=self.initial_transform_name,
                             capture_output=self.capture_output,
                             log_level=self.log_level,
@@ -509,8 +516,74 @@ class SingleAtlasSegmentation(Data):
                             )
                         )
 
-                self.segment_roi(im2.structure_sets[0][roi_name],
-                        strategy, self.steps[1], self.most_points2)
+            if (not reg_setup_only and
+                not self.segmentations[strategy][self.steps[0]]):
+                reg = self.registrations[strategy][self.steps[0]]
+                im2 = (reg.fixed_source if "push" == strategy
+                       else reg.moving_source)
+                self.segment_structure_set(im2.structure_sets[0],
+                        strategy, self.steps[0], self.most_points1)
+
+        if self.steps[1] in steps and (
+                not self.registrations[strategy][self.steps[1]] or
+                not self.segmentations[strategy][self.steps[1]]):
+
+            if not self.registrations[strategy][self.steps[1]]:
+
+                for roi_name in self.roi_names:
+                    im1, im2 = match_images(
+                            im1=self.im1,
+                            im2=self.im2,
+                            ss1_index = self.ss1_index,
+                            ss2_index = self.ss2_index,
+                            ss1=self.get_segmentation(strategy, 0, -1),
+                            ss2=self.ss2,
+                            ss1_name=self.ss1_name,
+                            ss2_name=self.ss2_name,
+                            roi_names={roi_name: self.roi_names[roi_name]},
+                            im2_crop_focus=roi_name,
+                            im2_crop_margins=self.roi_crop_margins.get(
+                                roi_name, self.default_roi_crop_margins),
+                            alignment=(roi_name if self.crop_to_match_size2
+                                       else False),
+                            voxel_size=self.voxel_size2,
+                            bands=self.roi_bands.get(
+                                roi_name, self.default_roi_bands)
+                            )
+
+                    fixed, moving = get_fixed_and_moving(im1, im2, strategy)
+
+                    self.registrations[strategy][self.steps[1]][roi_name] = (
+                            Registration(
+                                self.workdir / f"{strategy}2" / roi_name,
+                                fixed=fixed,
+                                moving=moving,
+                                pfiles=self.roi_pfiles.get(
+                                    roi_name, self.pfiles2_non_null),
+                                initial_alignment=roi_name,
+                                initial_transform_name=(
+                                    self.initial_transform_name),
+                                capture_output=self.capture_output,
+                                log_level=self.log_level,
+                                keep_tmp_dir=self.keep_tmp_dir,
+                                overwrite=force,
+                                )
+                            )
+
+            if (not reg_setup_only
+                and not self.segmentations[strategy][self.steps[1]]):
+                if self.initial_alignment:
+                    self.segmentations[strategy][self.steps[1]]\
+                            [self.initial_transform_name] = {}
+                for reg_step, pfile in self.pfiles2.items():
+                    self.segmentations[strategy][self.steps[1]][reg_step] = {}
+
+                for roi_name in self.roi_names:
+                    reg = self.registrations[strategy][self.steps[1]][roi_name]
+                    im2 = (reg.fixed_source if "push" == strategy
+                           else reg.moving_source)
+                    self.segment_roi(im2.structure_sets[0][roi_name], strategy,
+                                     self.steps[1], self.most_points2)
 
             for reg_step in list(self.segmentations[strategy][self.steps[1]]):
                 if self.segmentations[strategy][self.steps[1]][reg_step]:
@@ -558,7 +631,7 @@ class SingleAtlasSegmentation(Data):
             force=False):
         strategy = get_option(strategy, self.default_strategy, self.strategies)
         step = get_option(step, self.default_step, self.steps)
-        self.segment(strategy, step, force)
+        self.segment(strategy, step, force, reg_setup_only=True)
 
         if "local" == step:
             roi_names = sorted([key
@@ -670,6 +743,39 @@ class SingleAtlasSegmentation(Data):
 
         return df
 
+    def adjust_reg_files(self, strategies=None, step=None, roi_names=None,
+                         params_by_reg_step=None):
+        adjustments = {}
+        if not params_by_reg_step:
+            return adjustments
+        strategies = get_options(
+                strategies, self.default_strategy, self.strategies)
+        step = get_option(step, self.default_step, self.steps)
+        roi_names = roi_names or [None]
+
+        for strategy in strategies:
+            self.segment(strategy, step, True, reg_setup_only=True)
+
+        for reg_step, params in params_by_reg_step.items():
+            adjustments = {**adjustments,
+                           **{f"{step}_{reg_step}_{key}": value
+                              for key, value in params.items()}}
+            for strategy in strategies:
+                for roi_name in roi_names:
+                    self.get_registration(strategy, step, roi_name, False
+                                          ).adjust_file(reg_step, params)
+
+        return adjustments
+
+    def get_reg_steps(self, step=None):
+        step = get_option(step, None, self.steps)
+        initial_step = [self.initial_transform_name]
+        if step == self.steps[0]:
+            if not self.initial_alignment:
+                initial_step = []
+            return (initial_step + list(self.pfiles1))
+        else:
+            return (initial_step + list(self.pfiles2))
 
 def get_option(opt=None, fallback_opt=None, allowed_opts=None):
 
@@ -688,7 +794,7 @@ def get_option(opt=None, fallback_opt=None, allowed_opts=None):
 
 def get_options(opts=None, fallback_opts=None, allowed_opts=None):
 
-    if not is_list(allowed_opts) or not len(allowed_opts):
+    if not is_list(allowed_opts) or not allowed_opts:
         raise RuntimeError("No allowed options specified")
 
     if opts is True:
@@ -773,3 +879,83 @@ def get_structure_set_index(ss_index, im):
         and len(im.structure_sets) <= abs(ss_index)):
         return len(im.structure_sets) + ss_index
     return ss_index
+
+def get_sas_comparisons(pfiles1_variations=None, pfiles2_variations=None,
+                        **kwargs):
+
+    if not (pfiles1_variations or pfiles2_variations or kwargs):
+        return
+
+    sas_constant_kwargs = {}
+    sas_variable_kwargs = {}
+    comparison_kwargs = {}
+
+    sas_parameters = list(signature(SingleAtlasSegmentation).parameters)
+
+    for key, value in kwargs.items():
+        if key in sas_parameters:
+            if isinstance(value, list) and len(value) > 1:
+                sas_variable_kwargs[key] = value
+            else:
+                sas_constant_kwargs[key] = value
+        else:
+            comparison_kwargs[key] = value
+
+    sas_constant_kwargs["auto"] = True
+    sas_constant_kwargs["auto_reg_setup_only"] = True
+    sas_constant_kwargs["overwrite"] = True
+
+    if pfiles1_variations:
+        reg1_permutations = get_dict_permutations(
+                {reg_step: get_dict_permutations(pfiles1_variations[reg_step])
+                 for reg_step in pfiles1_variations})
+    else:
+        reg1_permutations = [{}]
+
+    if pfiles2_variations:
+        reg2_permutations = get_dict_permutations(
+                {reg_step: get_dict_permutations(pfiles2_variations[reg_step])
+                 for reg_step in pfiles2_variations})
+    else:
+        reg2_permutations = [{}]
+
+    sas_permutations = get_dict_permutations(sas_variable_kwargs)
+
+    df = None
+    for sas_permutation in sas_permutations:
+        sas = SingleAtlasSegmentation(**sas_constant_kwargs, **sas_permutation)
+        strategies = get_options(
+                sas.auto_strategies, sas.default_strategy, sas.strategies)
+        for reg1_permutation in reg1_permutations:
+            reg1_adjustments = sas.adjust_reg_files(
+                    strategies, sas.steps[0], None, reg1_permutation)
+            for reg2_permutation in reg2_permutations:
+                reg2_adjustments = sas.adjust_reg_files(
+                        strategies, sas.steps[1],
+                        sas.roi_names, reg2_permutation)
+                adjustments = {**sas_permutation, **reg1_adjustments,
+                               **reg2_adjustments}
+
+                df_comparison = sas.get_comparison(**comparison_kwargs)
+
+                if adjustments:
+                    df_permutation = pd.DataFrame(
+                            max(1, df_comparison.shape[0]) * [adjustments])
+                else:
+                    df_permutation = None
+
+                if df_permutation is None and df_comparison is None:
+                    continue
+                elif df_permutation is None:
+                    df_sas = df_comparison
+                elif df_comparison is None:
+                    df_sas = df_permutation
+                else:
+                    df_sas = pd.concat([df_permutation, df_comparison], axis=1)
+
+                if df is None:
+                    df = df_sas
+                else:
+                    df = pd.concat([df, df_sas], ignore_index=True)
+
+    return df
